/******************************************************************************
 *                                                                            *
 *                                Copyright by                                *
 *                                                                            *
 *                              Azoteq (Pty) Ltd                              *
 *                          Republic of South Africa                          *
 *                                                                            *
 *                           Tel: +27(0)21 863 0033                           *
 *                           E-mail: info@azoteq.com                          *
 *                                                                            *
 *============================================================================*
 * @file        IQS7222B.h                                                    *
 * @brief       This file contains all the necessary settings for the IQS7222B*
 *              and this file can be changed from the GUI or edited here      *
 * @author      JN. Lochner - Azoteq PTY Ltd                                  *
 * @version     v1.1                                                          *
 * @date        2023                                                          *
 *============================================================================*
 * @attention  Makes use of the following standard Arduino libraries:         *
 * - Arduino.h -> Included in IQS7222B.h, comes standard with Arduino         *
 * - Wire.h    -> Included in IQS7222B.h, comes standard with Arduino         *                                                                      
 *****************************************************************************/

#ifndef IQS7222B_h
#define IQS7222B_h

// Include Files
#include "Arduino.h"
#include "Wire.h"
#include "./inc/IQS7222B_addresses.h"

// Public Global Definitions
/* For use with Wire.h library. True argument with some functions closes the 
I2C communication window. */
#define STOP    true    
/* For use with Wire.h library. False argument with some functions keeps the 
I2C communication window open. */
#define RESTART false   

//Device Info
#define IQS7222B_PRODUCT_NUM            0x02BA

// Info Flags Byte Bits.
#define IQS7222B_ATI_ACTIVE_BIT		0
#define IQS7222B_ATI_ERROR_BIT	        1
#define IQS7222B_DEVICE_RESET_BIT	3
#define IQS7222B_POWER_EVENT_BIT_0      4
#define IQS7222B_POWER_EVENT_BIT_1      5
#define IQS7222B_NORMAL_POWER_BIT	0b00
#define IQS7222B_LOW_POWER_BIT		0b01
#define IQS7222B_ULP_BIT		0b10
#define IQS7222B_NORMAL_POWER_UPDATE	6
#define IQS7222B_GLOBAL_HALT    	7

// CHannel Proximity Bits
#define IQS7222B_CH0_PROX_BIT           0
#define IQS7222B_CH1_PROX_BIT           1
#define IQS7222B_CH2_PROX_BIT           2
#define IQS7222B_CH3_PROX_BIT           3
#define IQS7222B_CH4_PROX_BIT           4
#define IQS7222B_CH5_PROX_BIT           5
#define IQS7222B_CH6_PROX_BIT           6
#define IQS7222B_CH7_PROX_BIT           7
#define IQS7222B_CH8_PROX_BIT           0       // 8
#define IQS7222B_CH9_PROX_BIT           1       // 9
#define IQS7222B_CH10_PROX_BIT          2       // 10
#define IQS7222B_CH11_PROX_BIT          3       // 11
#define IQS7222B_CH12_PROX_BIT          4       // 12
#define IQS7222B_CH13_PROX_BIT          5       // 13
#define IQS7222B_CH14_PROX_BIT          6       // 14
#define IQS7222B_CH15_PROX_BIT          7       // 15
#define IQS7222B_CH16_PROX_BIT          0       
#define IQS7222B_CH17_PROX_BIT          1       
#define IQS7222B_CH18_PROX_BIT          2       
#define IQS7222B_CH19_PROX_BIT          3        

// CHannel Touch Bits
#define IQS7222B_CH0_TOUCH_BIT          0
#define IQS7222B_CH1_TOUCH_BIT          1
#define IQS7222B_CH2_TOUCH_BIT          2
#define IQS7222B_CH3_TOUCH_BIT          3
#define IQS7222B_CH4_TOUCH_BIT          4
#define IQS7222B_CH5_TOUCH_BIT          5
#define IQS7222B_CH6_TOUCH_BIT          6
#define IQS7222B_CH7_TOUCH_BIT          7
#define IQS7222B_CH8_TOUCH_BIT          0       // 8
#define IQS7222B_CH9_TOUCH_BIT          1       // 9
#define IQS7222B_CH10_TOUCH_BIT         2       // 10
#define IQS7222B_CH11_TOUCH_BIT         3       // 11
#define IQS7222B_CH12_TOUCH_BIT         4       // 12
#define IQS7222B_CH13_TOUCH_BIT         5       // 13
#define IQS7222B_CH14_TOUCH_BIT         6       // 14
#define IQS7222B_CH15_TOUCH_BIT         7       // 15
#define IQS7222B_CH16_TOUCH_BIT         0
#define IQS7222B_CH17_TOUCH_BIT         1
#define IQS7222B_CH18_TOUCH_BIT         2
#define IQS7222B_CH19_TOUCH_BIT         3

// Utility Bits
#define IQS7222B_ACK_RESET_BIT		        0
#define IQS7222B_SW_RESET_BIT		        1
#define IQS7222B_RE_ATI_BIT		        2
#define IQS7222B_RESEED_BIT		        3
#define IQS7222B_POWER_MODE_BIT_0               4       
#define IQS7222B_POWER_MODE_BIT_1               5
#define IQS7222B_INTERFACE_SELECT_BIT_0         6       
#define IQS7222B_INTERFACE_SELECT_BIT_1         7
#define IQS7222B_INTERFACE_I2C_STREAM_BIT	0b00
#define IQS7222B_INTERFACE_I2C_EVENT_BIT	0b01
#define IQS7222B_INTERFACE_I2C_STREAM_IN_TOUCH	0b10    

/* Defines and structs for IQS7222B states */
/**
* @brief  iqs7222b Init Enumeration.
*/
typedef enum {
        IQS7222B_INIT_NONE = (uint8_t) 0x00,
        IQS7222B_INIT_VERIFY_PRODUCT,
        IQS7222B_INIT_READ_RESET,
	IQS7222B_INIT_CHIP_RESET,
	IQS7222B_INIT_UPDATE_SETTINGS,
	IQS7222B_INIT_CHECK_RESET,
	IQS7222B_INIT_ACK_RESET,
	IQS7222B_INIT_ATI,
        IQS7222B_INIT_WAIT_FOR_ATI,
        IQS7222B_INIT_READ_DATA,
	IQS7222B_INIT_ACTIVATE_EVENT_MODE,
        IQS7222B_INIT_ACTIVATE_STREAM_IN_TOUCH_MODE,
	IQS7222B_INIT_DONE
} iqs7222b_init_e;

typedef enum {
        IQS7222B_STATE_NONE = (uint8_t) 0x00,
        IQS7222B_STATE_START,
        IQS7222B_STATE_INIT,
        IQS7222B_STATE_SW_RESET,
        IQS7222B_STATE_CHECK_RESET,
	IQS7222B_STATE_RUN,
} iqs7222b_state_e;

typedef enum {
        IQS7222B_CH0 = (uint8_t) 0x00,
        IQS7222B_CH1,
        IQS7222B_CH2,
        IQS7222B_CH3,
        IQS7222B_CH4,
        IQS7222B_CH5,
        IQS7222B_CH6,
        IQS7222B_CH7,
        IQS7222B_CH8,
        IQS7222B_CH9,
        IQS7222B_CH10,
        IQS7222B_CH11,
        IQS7222B_CH12,
        IQS7222B_CH13,
        IQS7222B_CH14,
        IQS7222B_CH15,
        IQS7222B_CH16,
        IQS7222B_CH17,
        IQS7222B_CH18,
        IQS7222B_CH19,
} iqs7222b_channel_e;

typedef enum 
{
        IQS7222B_CH_NONE = (uint8_t) 0x00,
        IQS7222B_CH_PROX,
        IQS7222B_CH_TOUCH,
        IQS7222B_CH_UNKNOWN,
} iqs7222b_ch_states;
typedef enum 
{
        IQS7222B_NORMAL_POWER = (uint8_t) 0x00,
        IQS7222B_LOW_POWER,
        IQS7222B_ULP,
        IQS7222B_HALT,
        IQS7222B_POWER_UNKNOWN
} iqs7222b_power_modes;

/* IQS7222B Memory map data variables, only save the data that might be used 
during program runtime */
#pragma pack(1)
typedef struct
{
	/* READ ONLY */			    //  I2C Addresses:
	uint8_t VERSION_DETAILS[20]; 	    // 	0x00 -> 0x09
	uint8_t SYSTEM_STATUS[2];           // 	0x10
	uint8_t EVENTS[2]; 		    // 	0x11
	uint8_t PROX_EVENT_STATES[4];       // 	0x12 -> 0x13
        uint8_t TOUCH_EVENT_STATES[4];      // 	0x14 -> 0x15

	/* READ WRITE */		    //  I2C Addresses:
	uint8_t SYSTEM_CONTROL[2]; 	    // 	0xD0
} IQS7222B_MEMORY_MAP;
#pragma pack(4)

#pragma pack(1)
typedef struct {
        iqs7222b_state_e      state;
        iqs7222b_init_e       init_state;
}iqs7222b_s;
#pragma pack(4)

// Class Prototype
class IQS7222B
{
public:
        // Public Constructors
        IQS7222B();

        // Public Device States
        iqs7222b_s iqs7222b_state;

        // Public Variables
        IQS7222B_MEMORY_MAP IQSMemoryMap;
        bool new_data_available;

        // Public Methods
        void begin(uint8_t deviceAddressIn, uint8_t readyPinIn);
        bool init(void);
        void run(void);
        void queueValueUpdates(void);
        bool readATIactive(void);
        uint16_t getProductNum(bool stopOrRestart);
        uint8_t getmajorVersion(bool stopOrRestart);
        uint8_t getminorVersion(bool stopOrRestart);
        void acknowledgeReset(bool stopOrRestart);
        void ReATI(bool stopOrRestart);
        void SW_Reset(bool stopOrRestart);
        void writeMM(bool stopOrRestart);
        void clearRDY(void);
        bool getRDYStatus(void);

        void setStreamMode(bool stopOrRestart);
        void setEventMode(bool stopOrRestart);
        void setStreamInTouchMode(bool stopOrRestart);

        void updateInfoFlags(bool stopOrRestart);
        iqs7222b_power_modes get_PowerMode(void);
        bool checkReset(void);

        bool channel_touchState(iqs7222b_channel_e channel);
        bool channel_proxState(iqs7222b_channel_e channel);

        void force_I2C_communication(void);

private:
        // Private Variables
        uint8_t _deviceAddress;

        // Private Methods
        void readRandomBytes(uint8_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart);
        void writeRandomBytes(uint8_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart);
        void writeRandomBytes16(uint16_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart);
        bool getBit(uint8_t data, uint8_t bit_number);
        uint8_t setBit(uint8_t data, uint8_t bit_number);
        uint8_t clearBit(uint8_t data, uint8_t bit_number);
};
#endif // IQS7222B_h  
