/******************************************************************************
 *                                                                            *
 *                                                                            *
 *                                Copyright by                                *
 *                                                                            *
 *                              Azoteq (Pty) Ltd                              *
 *                          Republic of South Africa                          *
 *                                                                            *
 *                           Tel: +27(0)21 863 0033                           *
 *                           E-mail: info@azoteq.com                          *
 *                                                                            *
 * ========================================================================== *
 * Refer to IQS7222B datasheet for more information, available here:          *
 * - https://www.azoteq.com/design/datasheets/                                *
 * ========================================================================== *
 *                       IQS7222B - Registers & Memory Map                    *
******************************************************************************/

#ifndef __IQS7222B_ADDRESSES_H
#define __IQS7222B_ADDRESSES_H

/* Device Information - Read Only */
#define IQS7222B_MM_PROD_NUM                  0x00
#define IQS7222B_MM_MAJOR_VERSION_NUM         0x01
#define IQS7222B_MM_MINOR_VERSION_NUM         0x02

/* DATA: 0x10 - 0x17 */
#define IQS7222B_MM_INFOFLAGS		          0x10
#define IQS7222B_MM_EVENTS                    0x11
#define IQS7222B_MM_PROX_EVENT_STATES_0_15    0x12
#define IQS7222B_MM_PROX_EVENT_STATES_16_19   0x13
#define IQS7222B_MM_TOUCH_EVENT_STATES_0_15   0x14
#define IQS7222B_MM_TOUCH_EVENT_STATES_16_19  0x15

/* CHANNEL COUNTS: 0x20 - 0x33 */
#define IQS7222B_MM_CHANNEL_0_COUNTS        0x20
#define IQS7222B_MM_CHANNEL_1_COUNTS        0x21
#define IQS7222B_MM_CHANNEL_2_COUNTS        0x22
#define IQS7222B_MM_CHANNEL_3_COUNTS        0x23
#define IQS7222B_MM_CHANNEL_4_COUNTS        0x24
#define IQS7222B_MM_CHANNEL_5_COUNTS        0x25
#define IQS7222B_MM_CHANNEL_6_COUNTS        0x26
#define IQS7222B_MM_CHANNEL_7_COUNTS        0x27
#define IQS7222B_MM_CHANNEL_8_COUNTS        0x28
#define IQS7222B_MM_CHANNEL_9_COUNTS        0x29
#define IQS7222B_MM_CHANNEL_10_COUNTS       0x2A
#define IQS7222B_MM_CHANNEL_11_COUNTS       0x2B
#define IQS7222B_MM_CHANNEL_12_COUNTS       0x2C
#define IQS7222B_MM_CHANNEL_13_COUNTS       0x2D
#define IQS7222B_MM_CHANNEL_14_COUNTS       0x2E
#define IQS7222B_MM_CHANNEL_15_COUNTS       0x2F
#define IQS7222B_MM_CHANNEL_16_COUNTS       0x30
#define IQS7222B_MM_CHANNEL_17_COUNTS       0x31
#define IQS7222B_MM_CHANNEL_18_COUNTS       0x32
#define IQS7222B_MM_CHANNEL_19_COUNTS       0x33

/* CHANNEL LTA: 0x40 - 0x53 */
#define IQS7222B_MM_CHANNEL_0_LTA           0x40
#define IQS7222B_MM_CHANNEL_1_LTA           0x41
#define IQS7222B_MM_CHANNEL_2_LTA           0x42
#define IQS7222B_MM_CHANNEL_3_LTA           0x43
#define IQS7222B_MM_CHANNEL_4_LTA           0x44
#define IQS7222B_MM_CHANNEL_5_LTA           0x45
#define IQS7222B_MM_CHANNEL_6_LTA           0x46
#define IQS7222B_MM_CHANNEL_7_LTA           0x47
#define IQS7222B_MM_CHANNEL_8_LTA           0x48
#define IQS7222B_MM_CHANNEL_9_LTA           0x49
#define IQS7222B_MM_CHANNEL_10_LTA          0x4A
#define IQS7222B_MM_CHANNEL_11_LTA          0x4B
#define IQS7222B_MM_CHANNEL_12_LTA          0x4C
#define IQS7222B_MM_CHANNEL_13_LTA          0x4D
#define IQS7222B_MM_CHANNEL_14_LTA          0x4E
#define IQS7222B_MM_CHANNEL_15_LTA          0x4F
#define IQS7222B_MM_CHANNEL_16_LTA          0x50
#define IQS7222B_MM_CHANNEL_17_LTA          0x51
#define IQS7222B_MM_CHANNEL_18_LTA          0x52
#define IQS7222B_MM_CHANNEL_19_LTA          0x53

/* CYCLE SETUP: 0x80 - 0x87 */
#define IQS7222B_MM_CYCLE_SETUP_0           0x8000
#define IQS7222B_MM_CYCLE_SETUP_1           0x8100
#define IQS7222B_MM_CYCLE_SETUP_2           0x8200
#define IQS7222B_MM_CYCLE_SETUP_3       	0x8300
#define IQS7222B_MM_CYCLE_SETUP_4           0x8400
#define IQS7222B_MM_CYCLE_SETUP_5           0x8500
#define IQS7222B_MM_CYCLE_SETUP_6           0x8600
#define IQS7222B_MM_CYCLE_SETUP_7           0x8700
#define IQS7222B_MM_CYCLE_SETUP_8           0x8800
#define IQS7222B_MM_CYCLE_SETUP_9           0x8900
#define IQS7222B_MM_GLOBAL_CYCLE_SETUP      0x8A00

/* BUTTON SETUP: 0x9000 - 0xA301 */
#define IQS7222B_MM_BUTTON_SETUP_0          0x9000
#define IQS7222B_MM_BUTTON_SETUP_1        	0x9100
#define IQS7222B_MM_BUTTON_SETUP_2          0x9200
#define IQS7222B_MM_BUTTON_SETUP_3          0x9300
#define IQS7222B_MM_BUTTON_SETUP_4          0x9400
#define IQS7222B_MM_BUTTON_SETUP_5       	0x9500
#define IQS7222B_MM_BUTTON_SETUP_6   		0x9600
#define IQS7222B_MM_BUTTON_SETUP_7          0x9700
#define IQS7222B_MM_BUTTON_SETUP_8          0x9800
#define IQS7222B_MM_BUTTON_SETUP_9          0x9900
#define IQS7222B_MM_BUTTON_SETUP_10         0x9A00
#define IQS7222B_MM_BUTTON_SETUP_11         0x9B00
#define IQS7222B_MM_BUTTON_SETUP_12         0x9C00
#define IQS7222B_MM_BUTTON_SETUP_13         0x9D00
#define IQS7222B_MM_BUTTON_SETUP_14         0x9E00
#define IQS7222B_MM_BUTTON_SETUP_15         0x9F00
#define IQS7222B_MM_BUTTON_SETUP_16         0xA000
#define IQS7222B_MM_BUTTON_SETUP_17         0xA100
#define IQS7222B_MM_BUTTON_SETUP_18         0xA200
#define IQS7222B_MM_BUTTON_SETUP_19         0xA300

/* CHANNEL SETUP: 0xB000 - 0xC301 */
#define IQS7222B_MM_CHANNEL_SETUP_0         0xB000
#define IQS7222B_MM_CHANNEL_SETUP_1         0xB100
#define IQS7222B_MM_CHANNEL_SETUP_2         0xB200
#define IQS7222B_MM_CHANNEL_SETUP_3         0xB300
#define IQS7222B_MM_CHANNEL_SETUP_4         0xB400
#define IQS7222B_MM_CHANNEL_SETUP_5         0xB500
#define IQS7222B_MM_CHANNEL_SETUP_6         0xB600
#define IQS7222B_MM_CHANNEL_SETUP_7         0xB700
#define IQS7222B_MM_CHANNEL_SETUP_8         0xB800
#define IQS7222B_MM_CHANNEL_SETUP_9         0xB900
#define IQS7222B_MM_CHANNEL_SETUP_10        0xBA00
#define IQS7222B_MM_CHANNEL_SETUP_11        0xBB00
#define IQS7222B_MM_CHANNEL_SETUP_12        0xBC00
#define IQS7222B_MM_CHANNEL_SETUP_13        0xBD00
#define IQS7222B_MM_CHANNEL_SETUP_14        0xBE00
#define IQS7222B_MM_CHANNEL_SETUP_15        0xBF00
#define IQS7222B_MM_CHANNEL_SETUP_16        0xC000
#define IQS7222B_MM_CHANNEL_SETUP_17        0xC100
#define IQS7222B_MM_CHANNEL_SETUP_18        0xC200
#define IQS7222B_MM_CHANNEL_SETUP_19        0xC300

/* FILTER BETAS: 0xC400 */
#define IQS7222B_MM_FILTER_BETAS            0xC400

/* PMU & SYSTEM SETTINGS: 0xD0 - 0xD9*/
#define IQS7222B_MM_CONTROL_SETTINGS        0xD0
#define IQS7222B_MM_ATI_ERROR_TIMEOUT       0xD1
#define IQS7222B_MM_ATI_REPORT_RATE         0xD2
#define IQS7222B_MM_NP_TIMEOUT         		0xD3
#define IQS7222B_MM_NP_REPORT_RATE         	0xD4
#define IQS7222B_MM_LP_TIMEOUT         		0xD5
#define IQS7222B_MM_LP_REPORT_RATE         	0xD6
#define IQS7222B_MM_ULP_NP_UPDATE_RATE      0xD7
#define IQS7222B_MM_ULP_REPORT_RATE         0xD8
#define IQS7222B_MM_TOUCH_PROX_TIMEOUT      0xD9

#endif /* __IQS7222B_ADDRESSES_H */